unit UnitMDC;
{
Unit: UnitMDC
Original Programming By  : Stuart "Stucuk" Carey
Description: This unit is based on the pdf document 'The MDC File Format' by Wolfgang Prinz

Note: This unit may be used freely as long as this header remains intact and credit is given.
      If you modify this unit update the changelog with ur changes.

Copyright  2004 Stuart Carey


Changelog

1.0
Revisions By Stucuk:
- Initial Version

}

interface

Uses Math3d, Unity_Types;

Const
MDCAnimationTime = 0.1; // 1 = 1 Second, basicaly how much time b4 u move fully into the next frame

type
TMDCFileHeader = record
ulIndent : array [0..3] of char; {IDPC}
ulVersion : Longword; {2}
ucName : array [0..63] of Char;
ulFlags,ulNumFrames,ulNumTags,ulNumSurfaces,
ulNumSkins,ulOffsetBorderFrames,ulOffsetTagNames,
ulOffsetTagFrames,ulOffsetSurfaces,ulOffsetEnd : Longword;
end;

TMDCFrame = record
bboxMin,bboxMax,localOrigin : TVector3f;
radius : single;
name : array [0..15] of char;
end;

TMDCTagName = record
ucName : array [0..63] of char;
end;

TMDCTagFrame = record
tpPosition, taAngle : TVector3si;
end;

TMDCSurfaceHeader = record
ulIndent : Longword;
ucName : array [0..63] of Char;
ulFlags,ulNumCompFrames,ulNumBaseFrames,ulNumShaders,
ulNumVertices,ulNumTriangles,ulOffsetTriangles,ulOffsetShaders,
ulOffsetTexCoords,ulOffsetBaseVerts,ulOffsetCompVerts,
ulOffsetFrameBaseFrames,ulOffsetFrameCompFrames,ulOffsetEnd : Longword;
end;

TMDCTriangle = record
ulVertex1,ulVertex2,ulVertex3 : longword;
end;

TMDCShader = record
ucName : array [0..63] of Char;
ulFlags : longword;
end;

TTexCoord = record
s,t : single;
end;

TVertex = record
x,y,z : single;
Normal : Byte;
end;

TVertex2 = record
x,y,z : single;
Normal : TVector3f;
end;

TCompVertex = record
x,y,z,n : Byte; //unsigned char
end;

TBaseFrame = Record
Vertexs : Array of TVertex;
offsetBaseVertices,
numBaseVertices : LongWord;
end;

TCompFrame = Record
Vertexs : Array of TCompVertex;
offsetCompVertices,
numCompVertices : LongWord;
end;


TTag = Record
Name : TMDCTagName;
Frame : TMDCTagFrame;
end;

TMDCTexture = Record
ShaderID  : Integer;
TextureID : Cardinal;
Shader    : String;
end;

TMDCSurface = Record // This will hold the information needed for 1 Surface
Header : TMDCSurfaceHeader;
OffsetStart : Integer;

Shaders : Array of TMDCShader;
Textures : Array Of TMDCTexture;

FrameBaseFrames,FrameCompFrames : Array of word;
Vertex,NVertex : Array of TVertex2; // Processed Vertexs!

BaseFrames : Array of TBaseFrame;
CompFrames : Array of TCompFrame;

Triangle : Array of TMDCTriangle;
TexCoord : Array of TTexCoord;

// Rendering Only
Data : Array of TRBSPVertex;

end;

TMDCFile = Record // This will hold the information needed for 1 MDC file
Header : TMDCFileHeader;

Frames : Array of TMDCFrame;
Surfaces : Array of TMDCSurface;
TagName : Array of TMDCTagName;
TagFrame : Array of Array of TMDCTagFrame;


// Own Definitions
Loaded: Boolean; //Incase its not loaded the program can check this!
Frame,NextFrame : Cardinal;
CurrentFrameTime : Single;
Origin : TVector3f;
Animate : Boolean;
end;

Procedure LoadMDC(Var Model : TMDCFile; Const Filename : String);
Procedure DrawMDC(Model : TMDCFile{; Position : TVector3f; Angles : TVector2f});

Procedure AddMDCShaders(MDC : TMDCFile);
Procedure AssignMDCShaders(MDC : TMDCFile);

Procedure ProcessVertexModel(Var Model : TMDCFile);

Procedure UpdateMDCFrame(Var MDCFile : TMDCFile; Const FrameTime : Single);

implementation

Uses Windows,Sysutils,OpenGL15,Unity_Global, Unity_Shader_System,Textures,
     Unity_FileSystem,Unity_MapSystem, UnitMDC_Normals;

Procedure ProcessVertex(Var Surface : TMDCSurface; Const NextVertex : Boolean; Model : TMDCFile; Frame : Cardinal);
var
x: integer;
VBase,VDelta,VLocalOrigin,
CompNormal,BaseNormal : TVector3f;
begin

If Model.Header.ulNumFrames > 0 then
If Surface.Header.ulNumVertices > 0 then
for x := 0 to Surface.Header.ulNumVertices-1 do
begin

VDelta       := SetVector(0,0,0);
VLocalOrigin := SetVector(0,0,0);
BaseNormal   := SetVector(0,0,0);
CompNormal   := SetVector(0,0,0);

VLocalOrigin := Model.Frames[frame].localOrigin;

If Surface.FrameCompFrames[frame] <> 65535 then
begin
//Note: Multiplying by 4 like the MDC PDF sais is too much, causes jerks
VDelta.X := (Surface.CompFrames[frame].Vertexs[x].X-127) * 1/64 * 3.1;
VDelta.Y := (Surface.CompFrames[frame].Vertexs[x].Y-127) * 1/64 * 3.1;
VDelta.Z := (Surface.CompFrames[frame].Vertexs[x].Z-127) * 1/64 * 3.1;
CompNormal.X :=  MDCNormals[Surface.CompFrames[Frame].Vertexs[x].N][0];
CompNormal.Y :=  MDCNormals[Surface.CompFrames[Frame].Vertexs[x].N][2];
CompNormal.Z := -MDCNormals[Surface.CompFrames[Frame].Vertexs[x].N][1];
end;

VBase.X := Surface.BaseFrames[Frame].Vertexs[x].X;
VBase.Y := Surface.BaseFrames[Frame].Vertexs[x].Y;
VBase.Z := Surface.BaseFrames[Frame].Vertexs[x].Z;

BaseNormal.X :=  MDCNormals[Surface.BaseFrames[Frame].Vertexs[x].Normal][0];
BaseNormal.Y :=  MDCNormals[Surface.BaseFrames[Frame].Vertexs[x].Normal][2];
BaseNormal.Z := -MDCNormals[Surface.BaseFrames[Frame].Vertexs[x].Normal][1];

if NextVertex then
begin
Surface.NVertex[x].X := VLocalOrigin.X + VBase.X + VDelta.X;
Surface.NVertex[x].Y := VLocalOrigin.Y + VBase.Y + VDelta.Y;
Surface.NVertex[x].Z := VLocalOrigin.Z + VBase.Z + VDelta.Z;
Surface.NVertex[x].Normal := DivVector2(AddVector(BaseNormal,CompNormal),2);
end
else
begin
Surface.Vertex[x].X := VLocalOrigin.X + VBase.X + VDelta.X;
Surface.Vertex[x].Y := VLocalOrigin.Y + VBase.Y + VDelta.Y;
Surface.Vertex[x].Z := VLocalOrigin.Z + VBase.Z + VDelta.Z;
Surface.Vertex[x].Normal := DivVector2(AddVector(BaseNormal,CompNormal),2);
end;
end;


end;

Procedure ProcessVertexModel(Var Model : TMDCFile);
var
x : integer;
begin

//Process Vertex's
for x := 0 to Model.Header.ulNumSurfaces-1 do
begin
ProcessVertex(Model.Surfaces[x],False,Model,Model.Frame);
ProcessVertex(Model.Surfaces[x],True,Model,Model.NextFrame);
end;

end;

Procedure LoadMDCSurface(Var Surface : TMDCSurface; Var F : File; Header : TMDCFileHeader);
Var
X,Y : integer;
S : SmallInt;
begin

SetLength(Surface.Shaders,Surface.Header.ulNumShaders);
SetLength(Surface.Textures,Surface.Header.ulNumShaders);
SetLength(Surface.FrameBaseFrames,Header.ulNumFrames);
SetLength(Surface.FrameCompFrames,Header.ulNumFrames);
SetLength(Surface.Vertex,Surface.Header.ulNumVertices);
SetLength(Surface.NVertex,Surface.Header.ulNumVertices);
SetLength(Surface.Triangle,Surface.Header.ulNumTriangles);
SetLength(Surface.TexCoord,Surface.Header.ulNumVertices);

SetLength(Surface.BaseFrames,Header.ulNumFrames);
SetLength(Surface.CompFrames,Header.ulNumFrames);

SetLength(Surface.Data,Surface.Header.ulNumTriangles*3);

Seek(F,Surface.OffsetStart + Surface.Header.ulOffsetShaders);

for x := 0 to Surface.Header.ulNumShaders-1 do
BlockRead(F,Surface.Shaders[x],SizeOf(TMDCShader));

Seek(F,Surface.OffsetStart + Surface.Header.ulOffsetFrameBaseFrames);

If Header.ulNumFrames > 0 then
for x := 0 to Header.ulNumFrames-1 do
BlockRead(F,Surface.FrameBaseFrames[x],SizeOf(word));

Seek(F,Surface.OffsetStart + Surface.Header.ulOffsetFrameCompFrames);

If Header.ulNumFrames > 0 then
for x := 0 to Header.ulNumFrames-1 do
BlockRead(F,Surface.FrameCompFrames[x],SizeOf(word));

for x := 0 to Header.ulNumFrames-1 do
begin
Surface.BaseFrames[x].offsetBaseVertices := Surface.OffsetStart + Surface.Header.ulOffsetBaseVerts + (Surface.FrameBaseFrames[x]*Surface.Header.ulNumVertices*8);
Surface.BaseFrames[x].numBaseVertices := Surface.Header.ulNumVertices;

SetLength(Surface.BaseFrames[x].Vertexs,Surface.Header.ulNumVertices);

Seek(F,Surface.BaseFrames[x].offsetBaseVertices);
for y := 0 to Surface.Header.ulNumVertices-1 do
begin
BlockRead(F,S,SizeOf(SmallInt));
Surface.BaseFrames[x].Vertexs[y].x := S/64;
BlockRead(F,S,SizeOf(SmallInt));
Surface.BaseFrames[x].Vertexs[y].Y := S/64;
BlockRead(F,S,SizeOf(SmallInt));
Surface.BaseFrames[x].Vertexs[y].Z := S/64;
BlockRead(F,Surface.BaseFrames[x].Vertexs[y].Normal,SizeOf(Word));
end;
end;


If Header.ulNumFrames > 0 then
for x := 0 to Header.ulNumFrames-1 do
begin
Surface.CompFrames[x].offsetCompVertices := Surface.OffsetStart + Surface.Header.ulOffsetCompVerts + (Surface.FrameCompFrames[x]*Surface.Header.ulNumVertices*4);

if Surface.FrameCompFrames[x] = 65535 then
begin
Surface.CompFrames[x].numCompVertices := 0;
continue;
end;

Surface.CompFrames[x].numCompVertices := Surface.Header.ulNumVertices;

SetLength(Surface.CompFrames[x].Vertexs,Surface.Header.ulNumVertices);

Seek(F,Surface.CompFrames[x].offsetCompVertices);
for y := 0 to Surface.Header.ulNumVertices-1 do
begin
BlockRead(F,Surface.CompFrames[x].Vertexs[y].X,SizeOf(ShortInt));
BlockRead(F,Surface.CompFrames[x].Vertexs[y].Y,SizeOf(ShortInt));
BlockRead(F,Surface.CompFrames[x].Vertexs[y].Z,SizeOf(ShortInt));
BlockRead(F,Surface.CompFrames[x].Vertexs[y].n,SizeOf(ShortInt));
end;
end;

Seek(F,Surface.OffsetStart + Surface.Header.ulOffsetTriangles);

for x := 0 to Surface.Header.ulNumTriangles-1 do
BlockRead(F,Surface.Triangle[x],SizeOf(TMDCTriangle));

Seek(F,Surface.OffsetStart + Surface.Header.ulOffsetTexCoords);

for x := 0 to Surface.Header.ulNumVertices-1 do
BlockRead(F,Surface.TexCoord[x],SizeOf(TTexCoord));

end;

Procedure LoadMDC(Var Model : TMDCFile; Const Filename : String);
Var
F : File;
X,Y,P : integer;
begin


AssignFile(f, filename);
Reset(F,1);

Model.Loaded := False;
Model.Frame := 0;
Model.CurrentFrameTime := 0;

Try
BlockRead(f,Model.Header,sizeof(TMDCFileHeader));

//Clear Arrays
SetLength(Model.Frames,0);
SetLength(Model.Surfaces,0);
SetLength(Model.TagName,0);
SetLength(Model.TagFrame,0,0);

If (Model.Header.ulIndent[0] <> 'I') or (Model.Header.ulIndent[1] <> 'D') or (Model.Header.ulIndent[2] <> 'P') or (Model.Header.ulIndent[3] <> 'C') or (Model.Header.ulVersion <> 2) then
begin
MessageBox(0,Pchar('Header Error:' + #13#13 + 'Filename: ' + Filename +#13 + 'ID: ' + Model.Header.ulIndent[0] + Model.Header.ulIndent[1] + Model.Header.ulIndent[2] + Model.Header.ulIndent[3] + #13 + 'Version: ' + inttostr(Model.Header.ulVersion)),'MDC Header Error',0);
Exit;
end;

//Set Array Lengths
SetLength(Model.Frames,Model.Header.ulNumFrames);
SetLength(Model.Surfaces,Model.Header.ulNumSurfaces);
SetLength(Model.TagName,Model.Header.ulNumTags);
SetLength(Model.TagFrame,Model.Header.ulNumTags,Model.Header.ulNumFrames);

If Model.Header.ulNumFrames > 1 then
Model.NextFrame := 1
else
Model.NextFrame := 0;

Seek(F,Model.Header.ulOffsetBorderFrames);
for x := 0 to Model.Header.ulNumFrames-1 do
BlockRead(f,Model.Frames[x],sizeof(TMDCFrame));

Seek(F,Model.Header.ulOffsetSurfaces);
P := Model.Header.ulOffsetSurfaces;
for x := 0 to Model.Header.ulNumSurfaces-1 do
begin
BlockRead(f,Model.Surfaces[x].Header,sizeof(TMDCSurfaceHeader));

If Model.Surfaces[x].Header.ulIndent <> 7 then
begin // Assume bad surface
Model.Header.ulNumSurfaces := x;
Break;
end;

Model.Surfaces[x].OffsetStart := P;
LoadMDCSurface(Model.Surfaces[x],F,Model.Header);
P := P + Model.Surfaces[x].Header.ulOffsetEnd;

Seek(F,P);
end;

Seek(F,Model.Header.ulOffsetTagNames);
If model.Header.ulNumTags > 0 then
for x := 0 to Model.Header.ulNumTags-1 do
BlockRead(f,Model.TagName[x],sizeof(TMDCTagName));

Seek(F,Model.Header.ulOffsetTagFrames);
If model.Header.ulNumTags > 0 then
for x := 0 to Model.Header.ulNumTags-1 do
for y := 0 to Model.Header.ulNumFrames-1 do
BlockRead(f,Model.TagFrame[x][y],sizeof(TMDCTagFrame));

ProcessVertexModel(Model);

Model.Loaded := True;

Except
End;
Closefile(f);
end;

Procedure DrawSurface(Surface : TMDCSurface; TextureID : Integer);
var
x : integer;
begin

glBindTexture(GL_TEXTURE_2D,TextureID);

glPolygonOffset(0.1,0.1);

glVertexPointer(3, GL_FLOAT, sizeof(TRBSPVertex), @Surface.Data[0].Position);

glTexCoordPointer(2, GL_FLOAT, sizeof(TRBSPVertex), @Surface.Data[0].TextureCoord);
glDrawArrays(GL_TRIANGLES, 0, Surface.Header.ulNumTriangles*3);
       {
glBegin(GL_TRIANGLES);
for x := 0 to Surface.Header.ulNumTriangles-1 do
begin
  glTexCoord2f(Surface.Data[x*3].TextureCoord.X, Surface.Data[x*3].TextureCoord.Y);
  glVertex3f(Surface.Data[x*3].Position.X, Surface.Data[x*3].Position.Y, Surface.Data[x*3].Position.Z);

  glTexCoord2f(Surface.Data[x*3+1].TextureCoord.X, Surface.Data[x*3+1].TextureCoord.Y);
  glVertex3f(Surface.Data[x*3+1].Position.X, Surface.Data[x*3+1].Position.Y, Surface.Data[x*3+1].Position.Z);

  glTexCoord2f(Surface.Data[x*3+2].TextureCoord.X, Surface.Data[x*3+2].TextureCoord.Y);
  glVertex3f(Surface.Data[x*3+2].Position.X, Surface.Data[x*3+2].Position.Y, Surface.Data[x*3+2].Position.Z);
end;
glEnd;  }

end;

procedure DrawSurfaceShader(Surface : TMDCSurface; ShaderID : Integer);
var
l,x,Count : Integer;
Shader : TShader;
begin

  if ShaderID = -1 then
  begin
  Exit;
  end;

  Count := Surface.Header.ulNumTriangles*3;

  Shader := Engine.ShaderManagerModel[ShaderId];

  if shader.Cull = GL_NONE then
    glDisable(GL_CULL_FACE)
  else begin
    glCullFace(shader.cull);
  end;

  try

  // Give OpenGL our vertices to use for vertex arrays

  Engine.ShaderManagerModel.SetVertexCoords_Shader(ShaderId,Count,Surface.Data);

  glVertexPointer(3, GL_FLOAT, sizeof(TRBSPVertex), @Surface.Data[0].Position);

  for l := 0 to Shader.numOfLayers-1 do
  begin
  Engine.ShaderManagerModel.SetTextureCoordinates_Shader(ShaderId, l, Count,Surface.Data);


  If (Shader.Layers[l].TextureId[Shader.Layers[l].curTexture] > 0) then
  glBindTexture(GL_TEXTURE_2D, Shader.Layers[l].TextureId[Shader.Layers[l].curTexture])
  else
  begin
  glBindTexture(GL_TEXTURE_2D, 0);
//  Inc(Engine.Map.MissingTextures);
  end;


  Engine.ShaderManagerModel.SetState(ShaderId,l);

  glTexCoordPointer(2, GL_FLOAT, sizeof(TRBSPVertex), @Surface.Data[0].TextureCoord);
  glDrawArrays(GL_TRIANGLES, 0, Surface.Header.ulNumTriangles*3);

          {
  glBegin(GL_TRIANGLES);
  for x := 0 to Surface.Header.ulNumTriangles-1 do
  begin
  glTexCoord2f(Surface.Data[x*3].TextureCoord.X, Surface.Data[x*3].TextureCoord.Y);
  glVertex3f(Surface.Data[x*3].Position.X, Surface.Data[x*3].Position.Y, Surface.Data[x*3].Position.Z);

  glTexCoord2f(Surface.Data[x*3+1].TextureCoord.X, Surface.Data[x*3+1].TextureCoord.Y);
  glVertex3f(Surface.Data[x*3+1].Position.X, Surface.Data[x*3+1].Position.Y, Surface.Data[x*3+1].Position.Z);

  glTexCoord2f(Surface.Data[x*3+2].TextureCoord.X, Surface.Data[x*3+2].TextureCoord.Y);
  glVertex3f(Surface.Data[x*3+2].Position.X, Surface.Data[x*3+2].Position.Y, Surface.Data[x*3+2].Position.Z);
  end;
  glEnd; }

  end;

  if shader.Cull = GL_NONE then
    glEnable(GL_CULL_FACE);

  SetDefaults;

    except

    end;
end;

Function GetTransitionVertex(Surface  : TMDCSurface; ID : Integer; CurrentFrameTime : Single) : TVertex2;
begin

Result.X := Surface.Vertex[ID].x - ((Surface.Vertex[ID].x - Surface.NVertex[ID].x)* CurrentFrameTime/MDCAnimationTime);
Result.Y := Surface.Vertex[ID].y - ((Surface.Vertex[ID].y - Surface.NVertex[ID].y)* CurrentFrameTime/MDCAnimationTime);
Result.Z := Surface.Vertex[ID].z - ((Surface.Vertex[ID].z - Surface.NVertex[ID].z)* CurrentFrameTime/MDCAnimationTime);
Result.Normal := Surface.Vertex[ID].Normal;

end;

Procedure DrawMDCSurface(Surface : TMDCSurface; Origin : TVector3f; CurrentFrameTime : Single);
var
x,c : integer;
Vertex : TVertex2;
begin

glColor3f(1,1,1);

SetDefaults;

c := 0;
for x := 0 to Surface.Header.ulNumTriangles-1 do
begin
Vertex                         := GetTransitionVertex(Surface,Surface.Triangle[x].ulVertex1,CurrentFrameTime);
Surface.Data[c].Position       := AddVector(SetVector(Vertex.x,Vertex.z,-Vertex.y),Origin);
Surface.Data[c].Normal         := Vertex.Normal;
Surface.Data[c].TextureCoord.X :=   Surface.TexCoord[Surface.Triangle[x].ulVertex1].s;
Surface.Data[c].TextureCoord.Y := 1-Surface.TexCoord[Surface.Triangle[x].ulVertex1].t;

Vertex                           := GetTransitionVertex(Surface,Surface.Triangle[x].ulVertex2,CurrentFrameTime);
Surface.Data[c+1].Position       := AddVector(SetVector(Vertex.x,Vertex.z,-Vertex.y),Origin);
Surface.Data[c+1].Normal         := Vertex.Normal;
Surface.Data[c+1].TextureCoord.X :=   Surface.TexCoord[Surface.Triangle[x].ulVertex2].s;
Surface.Data[c+1].TextureCoord.Y := 1-Surface.TexCoord[Surface.Triangle[x].ulVertex2].t;

Vertex                           := GetTransitionVertex(Surface,Surface.Triangle[x].ulVertex3,CurrentFrameTime);
Surface.Data[c+2].Position       := AddVector(SetVector(Vertex.x,Vertex.z,-Vertex.y),Origin);
Surface.Data[c+2].Normal         := Vertex.Normal;
Surface.Data[c+2].TextureCoord.X :=   Surface.TexCoord[Surface.Triangle[x].ulVertex3].s;
Surface.Data[c+2].TextureCoord.Y := 1-Surface.TexCoord[Surface.Triangle[x].ulVertex3].t;

inc(c,3);
{If Surface.Textures[0].ShaderID > -1 then
DrawTriangleShader(GetTransitionVertex(Surface,Surface.Triangle[x].ulVertex1,CurrentFrameTime),GetTransitionVertex(Surface,Surface.Triangle[x].ulVertex2,CurrentFrameTime),GetTransitionVertex(Surface,Surface.Triangle[x].ulVertex3,CurrentFrameTime),Surface.TexCoord[Surface.Triangle[x].ulVertex1],Surface.TexCoord[Surface.Triangle[x].ulVertex2],Surface.TexCoord[Surface.Triangle[x].ulVertex3],Surface.Textures[0].ShaderID,Origin)
else
DrawTriangle(GetTransitionVertex(Surface,Surface.Triangle[x].ulVertex1,CurrentFrameTime),GetTransitionVertex(Surface,Surface.Triangle[x].ulVertex2,CurrentFrameTime),GetTransitionVertex(Surface,Surface.Triangle[x].ulVertex3,CurrentFrameTime),Surface.TexCoord[Surface.Triangle[x].ulVertex1],Surface.TexCoord[Surface.Triangle[x].ulVertex2],Surface.TexCoord[Surface.Triangle[x].ulVertex3],Surface.Textures[0].TextureID,Origin);
}
end;

If Surface.Textures[0].ShaderID > -1 then
DrawSurfaceShader(Surface,Surface.Textures[0].ShaderID)
else
DrawSurface(Surface,Surface.Textures[0].TextureID);

end;

Procedure DrawMDC(Model : TMDCFile);
var
x : integer;
begin

for x := 0 to Model.Header.ulNumSurfaces-1 do
DrawMDCSurface(Model.Surfaces[x],Model.Origin,Model.CurrentFrameTime);

end;

Procedure AddMDCShader(Surface : TMDCSurface);
var
x : integer;
begin

for x := 0 to Surface.Header.ulNumShaders-1 do
begin
Surface.Textures[x].Shader := Engine.ShaderManagerModel.GetTextureNameInShaderFormat(String(Surface.Shaders[x].ucName));
Engine.ShaderManagerModel.AddToShaderManager(Surface.Textures[x].Shader);
end;

end;

Procedure AddMDCShaders(MDC : TMDCFile);
var
x : integer;
begin

for x := 0 to MDC.Header.ulNumSurfaces-1 do
AddMDCShader(MDC.Surfaces[x]);

end;

Procedure AssignMDCShader(Var Surface : TMDCSurface);
var
x : integer;
begin

for x := 0 to Surface.Header.ulNumShaders-1 do
begin
Surface.Textures[x].ShaderID := Engine.ShaderManagerModel.IndexOf(Surface.Textures[x].Shader);
If Surface.Textures[x].ShaderID > -1 then
Surface.Textures[x].TextureID := Engine.ShaderManagerModel.Items[Surface.Textures[x].ShaderID].qerId
else
LoadQuakeTexture(Engine.Main.GAME_FOLDER + '\' + ENGINE_FILES_DIR + ExtractFileDir(String(Surface.Shaders[x].ucName)) + '\',ExtractFileName(String(Surface.Shaders[x].ucName)),Surface.Textures[x].TextureID,False,False);
end;

end;

Procedure AssignMDCShaders(MDC : TMDCFile);
var
x : integer;
begin

for x := 0 to MDC.Header.ulNumSurfaces-1 do
AssignMDCShader(MDC.Surfaces[x]);

end;

Procedure UpdateMDCFrame(Var MDCFile : TMDCFile; Const FrameTime : Single);
begin

If (not MDCFile.Animate) or (MDCFile.Header.ulNumFrames < 2) then exit;

If MDCFile.CurrentFrameTime >= MDCAnimationTime then
begin

repeat
MDCFile.CurrentFrameTime := MDCFile.CurrentFrameTime-MDCAnimationTime;
MDCFile.Frame := MDCFile.NextFrame;

If MDCFile.NextFrame + 1 >= MDCFile.Header.ulNumFrames then
MDCFile.NextFrame := 0
else
Inc(MDCFile.NextFrame);
until MDCFile.CurrentFrameTime < MDCAnimationTime;

ProcessVertexModel(MDCFile);
end;

MDCFile.CurrentFrameTime := MDCFile.CurrentFrameTime + FrameTime;

end;

end.
