unit math3d;

interface

type TVector3f = record
  X, Y, Z : single;
end;

type TVector3i = record
  X, Y, Z : integer;
end;

type TVector3b = record
  R, G, B : byte;
end;

type TVector3b2 = record
  X, Y, Z : byte;
end;

type TVector3si = record
  X, Y, Z : smallint;
end;

type TVector3l = record
  X, Y, Z : longword;
end;

type TVector2f = record
  X, Y : single;
end;

type TVector2i = record
  X, Y : Integer;
end;

Type TVector4f = record
  X, Y, Z, W : single;
end;

  PSmallIntArray = ^TSmallIntArray;
  TSmallIntArray = array[0..MAXINT shr 2] of SmallInt;

  PCardinalArray = ^TCardinalArray;
  TCardinalArray = array[0..MAXINT shr 3] of Cardinal;

// TGLPixel32
   //
   TGLPixel32 = packed record
      r, g, b, a : Byte;
   end;

   PGLPixel32Array = ^TGLPixel32Array;
   TGLPixel32Array = array of TGLPixel32;

   // General arrays
   PByteArray = ^TByteArray;
   TByteArray = array[0..32767] of Byte;

const
  M_PI = 3.1415926535897932384626433832795;		// matches value in gcc v2 math.h
  M_TWO_PI = 6.283185307179586476925286766559;
  M_180_PI = 57.295779513082320876798154814105;
  M_PI_180 = 0.017453292519943295769236907684886;


function VectorDistance(const v1, v2 : TVector3f) : Single;
function DEG2RAD(a : single) : single;
function RAD2DEG(a : single) : single;
function Normalize(var v : TVector3f) : single;
function Normalize2(v : TVector3f) : TVector3f;
function ScaleVector(v : TVector3f; s : single) : TVector3f;
function ScaleVector3f(v,s : TVector3f) : TVector3f;
function AddVector(v1, v2 : TVector3f) : TVector3f;
function SubtractVector(v1, v2 : TVector3f) : TVector3f;
function DotProduct(v1, v2 : TVector3f) : single;
function InvertVector(v : TVector3f) : TVector3f;
function planeDistance(point, PlaneNormal : TVector3f; PlaneDistance : single) : single;
function SetVector(x, y, z : single) : TVector3f;
function SetVector4f(x, y, z, w : single) : TVector4f;
Function TVector4fToVector3f(V : TVector4f) : Tvector3f;
function SetVectori(x, y, z : integer) : TVector3i;
function CrossProduct(const V1, V2: TVector3f): TVector3f;
function ClassifyPoint(point, PlaneNormal : TVector3f; PlaneDistance : single) : single; overload;
function ClassifyPoint(PlaneNormal : TVector3f; PlaneDistance : single) : single; overload;
function ClassifyPoint(Point, PointOnPlane, PlaneNormal : TVector3f) : single; overload;
procedure VecToAngles(const _In: TVector3f ; var angles: TVector3f);

Function IsPointOn3dLine(LS,LE,Point : TVector3f) : Boolean;
Function ISBoundsOnLine(BMin,Bmax, LS,LE : TVector3f) : Boolean;
Function GetCenterOfBounds(Bmax,Bmin : TVector3f) : TVector3f;
Function GetClosestPointOn3dLine(LS,LE,Point : TVector3f) : TVector3f;
Function DivVector(V1,V2 : TVector3f) : TVector3f;
Function DivVector2(V1 : TVector3f; V2 : Extended) : TVector3f;
Function AbsVector(X : TVector3f) : TVector3f;
function GetPositionInfrontFromAngles(Distance : single; X_Angle,Y_Angle : Single) : TVector3f;
Function GetPointOnLineUsingK(P1,P2 : TVector3f; K : Extended) : TVector3f;

Function ConvertHLAngleToHelios(Angle : Single) : Single;
Function SqrtVector(V : TVector3f) : TVector3f;
Function Min3f(V,V2 : TVector3f) : Boolean;
Function Max3f(V,V2 : TVector3f) : Boolean;
Function Min3f2(V,V2 : TVector3f) : TVector3f;
Function Max3f2(V,V2 : TVector3f) : TVector3f;

Function IsVectorsDifferent(V1,V2 : TVector3f) : Boolean;

implementation

uses math;

function VectorDistance(const v1, v2 : TVector3f) : Single; register;
// EAX contains address of v1
// EDX contains highest of v2
// Result  is passed on the stack
asm
      FLD  DWORD PTR [EAX]
      FSUB DWORD PTR [EDX]
      FMUL ST, ST
      FLD  DWORD PTR [EAX+4]
      FSUB DWORD PTR [EDX+4]
      FMUL ST, ST
      FADD
      FLD  DWORD PTR [EAX+8]
      FSUB DWORD PTR [EDX+8]
      FMUL ST, ST
      FADD
      FSQRT
end;

function SetVector(x, y, z : single) : TVector3f;
begin
  result.x := x;
  result.y := y;
  result.z := z;
end;

function SetVector4f(x, y, z, w : single) : TVector4f;
begin
  result.x := x;
  result.y := y;
  result.z := z;
  Result.w := w;
end;

Function TVector4fToVector3f(V : TVector4f) : Tvector3f;
begin
  result.x := V.x;
  result.y := V.y;
  result.z := V.z;
end;

function SetVectori(x, y, z : integer) : TVector3i;
begin
  result.x := x;
  result.y := y;
  result.z := z;
end;

function DEG2RAD(a : single) : single;
begin
  result :=  a*M_PI_180;
end;

function RAD2DEG(a : single) : single;
begin
  result := a*M_180_PI;
end;

function Normalize(var v : TVector3f) : single;
var l : single;
begin
  l := sqrt(v.x*v.x+v.y*v.y+v.z*v.z);

  if (l > 0) then begin
    v.x := v.x/l;
    v.y := v.y/l;
    v.z := v.Z/l;
  end;
  result := l;  
end;

function Normalize2(v : TVector3f) : TVector3f;
var l : single;
begin
  l := sqrt(v.x*v.x+v.y*v.y+v.z*v.z);

  if (l > 0) then begin
    v.x := v.x/l;
    v.y := v.y/l;
    v.z := v.Z/l;
  end;
  result := v;
end;

function ScaleVector(v : TVector3f; s : single) : TVector3f;
begin
  with result do begin
    x := v.x * s;
    y := v.y * s;
    z := v.z * s;
  end;
end;

function ScaleVector3f(v,s : TVector3f) : TVector3f;
begin
  with result do begin
    x := v.x * s.x;
    y := v.y * s.y;
    z := v.z * s.z;
  end;
end;

function AddVector(v1, v2 : TVector3f) : TVector3f;
begin
  with result do begin
    x := v1.x + v2.x;
    y := v1.y + v2.y;
    z := v1.z + v2.z;
  end;
end;

function SubtractVector(v1, v2 : TVector3f) : TVector3f;
begin
  with result do begin
    x := v1.x - v2.x;
    y := v1.y - v2.y;
    z := v1.z - v2.z;
  end;
end;

function DotProduct(v1, v2 : TVector3f) : single;
begin
  Result := v1.X*v2.X + v1.y*v2.y + v1.z*v2.z;
end;

function InvertVector(v : TVector3f) : TVector3f;
begin
  with result do begin
    x := -v.x;
    y := -v.y;
    z := -v.z;
  end;
end;

function CrossProduct(const V1, V2: TVector3f): TVector3f;
begin
   Result.X:=V1.Y * V2.z - V1.z * V2.y;
   Result.Y:=V1.z * V2.x - V1.x * V2.z;
//   Result.Z:=V1.x * V2.y - V1.x * V2.x;
   Result.Z:=V1.x * V2.y - V1.y * V2.x;
end;

function planeDistance(point, PlaneNormal : TVector3f; PlaneDistance : single) : single;
begin
  result := (planeNormal.x * point.x +
	     planeNormal.y * point.y +
	     planeNormal.z * point.z) - PlaneDistance;
end;

// point = test point
// p0 = point on plane = VectorScale(Plane.Normal, -Plane.d)
// pN = plane normal
function ClassifyPoint(point, PlaneNormal : TVector3f; PlaneDistance : single) : single;
var pointOnPlane, tempVect, dir : TVector3f;
    d : single;
begin
 pointOnPlane := ScaleVector(PlaneNormal, -PlaneDistance);
 TempVect.x := pointOnPlane.x - point.x;
 TempVect.y := pointOnPlane.y - point.y;
 TempVect.z := pointOnPlane.z - point.z;
 dir := TempVect;
 d := DotProduct(dir, PlaneNormal);
 if (d < -0.001) then
   result := 1
 else if (d > 0.001) then
   result := -1
 else
   result := 0;
end;

function ClassifyPoint(PlaneNormal : TVector3f; PlaneDistance : single) : single;
var pointOnPlane, tempVect, dir, point : TVector3f;
    d : single;
begin
 point := SetVector(0,0,0);
 pointOnPlane := ScaleVector(PlaneNormal, -PlaneDistance);
 TempVect.x := pointOnPlane.x - point.x;
 TempVect.y := pointOnPlane.y - point.y;
 TempVect.z := pointOnPlane.z - point.z;
 dir := TempVect;
 d := DotProduct(dir, PlaneNormal);
 if (d < -0.001) then
   result := 1
 else if (d > 0.001) then
   result := -1
 else
   result := 0;
end;

function ClassifyPoint(Point, PointOnPlane, PlaneNormal : TVector3f) : single; overload;
var tempVect, dir : TVector3f;
    d : single;
begin
 TempVect.x := pointOnPlane.x - point.x;
 TempVect.y := pointOnPlane.y - point.y;
 TempVect.z := pointOnPlane.z - point.z;
 dir := TempVect;
 d := DotProduct(dir, PlaneNormal);
 if (d < -0.001) then
   result := 1
 else if (d > 0.001) then
   result := -1
 else
   result := 0;
end;

// From Q2 To Delphi Project
procedure VecToAngles(const _In: TVector3f ; var angles: TVector3f);
var
  forward_: Single;
  yaw, pitch: Single;
begin
  if (_In.z = 0) and (_In.x = 0) then
  begin
    yaw := 0;
    if (_In.y > 0) then
      pitch := 90
    else
      pitch := 270;
  end
  else
  begin
    if (_In.x <> 0) then
      yaw := RadToDeg(ArcTan2(_In.z, _In.x));// * 180 / M_PI
{    else if (_In.z > 0) then
      yaw := 90
    else
      yaw := -90; }
 {   if (yaw < -90) then
      yaw := yaw + 180;
    if (yaw > 90) then
      yaw := yaw - 180;  }



    forward_ := sqrt(_In.x*_In.x + _In.z*_In.z);
    pitch := RadToDeg(ArcTan2(_In.y, forward_));// * 180 / M_PI;
{    if (pitch < 0) then
      pitch := pitch + 360;}

{    if (pitch < -90) then
      pitch := pitch + 180;
    if (yaw > 90) then
      pitch := pitch - 180; }
  end;

  angles.x := {-}pitch;
  angles.y := yaw;
  angles.z := 0;
end;

Function DivVector(V1,V2 : TVector3f) : TVector3f;
begin

Result := SetVector(0,0,0);

if V2.X <> 0 then
Result.X := V1.X / V2.X;

if V2.Y <> 0 then
Result.Y := V1.Y / V2.Y;

if V2.Z <> 0 then
Result.Z := V1.Z / V2.Z;

end;

Function DivVector2(V1 : TVector3f; V2 : Extended) : TVector3f;
begin

Result := SetVector(0,0,0);

if V2 <> 0 then
Result.X := V1.X / V2;

if V2 <> 0 then
Result.Y := V1.Y / V2;

if V2 <> 0 then
Result.Z := V1.Z / V2;

end;

Function ISValidPoint(Point : Extended): Boolean;
begin

Result := False;

if Point > 1 then
Exit;

if Point < 0 then
Exit;

Result := True;

end;

Function IsPointOn3dLine(LS,LE,Point : TVector3f) : Boolean;
var
K : TVector3f;
begin

K := DivVector(SubtractVector(Point,LS),SubtractVector(LE,LS));

Result := false;

if not ISValidPoint(K.X) then
exit;

if not ISValidPoint(K.Y) then
exit;

if not ISValidPoint(K.Z) then
exit;

If (K.X = K.Y) and (K.Z = K.X) then
Result := true;


{
K := (Point-LS)/(LE-LS)


P = P1 + k(P2-P1)
P-P1 = k(P2-P1)
(P-P1)/(P2-P1) = k    }
end;

Function GetClosestPointOn3dLine(LS,LE,Point : TVector3f) : TVector3f;
var
K,Kx,Ky,Kz : TVector3f;
Dx,Dy,Dz : Extended;
begin

K := DivVector(SubtractVector(Point,LS),SubtractVector(LE,LS));

// K is 0 < k < 1, so we check here and make sure it is
if K.X > 1 then
K.X := 1
else
if K.X < 0 then
K.X := 0;

if K.Y > 1 then
K.Y := 1
else
if K.Y < 0 then
K.Y := 0;

if K.Z > 1 then
K.Z := 1
else
if K.Z < 0 then
K.Z := 0;

Kx := AddVector(LS ,(ScaleVector(SubtractVector(LE,LS),K.X)));
Ky := AddVector(LS ,(ScaleVector(SubtractVector(LE,LS),K.Y)));
Kz := AddVector(LS ,(ScaleVector(SubtractVector(LE,LS),K.Z)));

Dx := VectorDistance(Point,Kx);
Dy := VectorDistance(Point,Ky);
Dz := VectorDistance(Point,Kz);

Result := LS;

If (Dx <= Dy) and (Dx <= Dz) then
begin
Result := Kx;
exit;
end;

If (Dy <= Dx) and (Dy <= Dz) then
begin
Result := Ky;
exit;
end;

If (Dz <= Dx) and (Dz <= Dy) then
begin
Result := Kz;
exit;
end;

end;

Function GetCenterOfBounds(Bmax,Bmin : TVector3f) : TVector3f;
begin
Result := AddVector(DivVector2(SubtractVector(Bmax,Bmin),2),BMin);
end;

Function ISBoundsOnLine(BMin,Bmax, LS,LE : TVector3f) : Boolean;
Var
V : TVector3f;
begin

V := GetClosestPointOn3dLine(LS,LE,GetCenterOfBounds(Bmax,Bmin));

Result := False;

If (V.X < BMin.X) or (V.X > BMax.X) then
exit;

If (V.Y < BMin.Y) or (V.Y > BMax.Y) then
exit;

If (V.Z < BMin.Z) or (V.Z > BMax.Z) then
exit;

Result := True;

end;

Function AbsVector(X : TVector3f) : TVector3f;
begin

Result.X := Abs(X.X);
Result.Y := Abs(X.Y);
Result.Z := Abs(X.Z);

end;

function GetPositionInfrontFromAngles(Distance : single; X_Angle,Y_Angle : Single) : TVector3f;
var dir, versor, desp, temp,Velos : TVector3f;
    rad_yaw, vel,Speed : single;
label MouseAction;
begin
  Speed := 100;
  dir    := SetVector(0,0,0);
  versor := SetVector( 0,0,0);
  Velos := SetVector(0,0,0);

  begin
  versor.z := versor.z-1.0;
  versor.y := versor.y+1.0;
  end;


  rad_yaw := DEG2RAD(y_angle);

  // move

  if (versor.z <> 0) then begin
    dir.x := dir.x -  versor.z*Sin(rad_yaw);
    dir.z := dir.z +  versor.z*Cos(rad_yaw);
  end;

  IF X_Angle > 90 then
  X_Angle := 90
  else
  IF X_Angle < -90 then
  X_Angle := -90;

  if X_angle = 90 then
  rad_yaw := DEG2RAD(-x_angle)
  else
  if X_angle = -90 then
  rad_yaw := DEG2RAD(-x_angle)
  else
  rad_yaw := DEG2RAD(x_angle);
  // move
  if (versor.y <> 0) then
  dir.y := (dir.y -  versor.y*Tan(rad_yaw));

  rad_yaw := DEG2RAD(y_angle);

  if (Normalize(versor) > 0) then begin
    Velos.x := dir.x*speed*2000;
    Velos.y := dir.y*speed*2000;
    Velos.z := dir.z*speed*2000;
  end;

  // Speed
  temp := Velos;
  vel := Normalize(temp);

  Velos.x := Velos.x - temp.x*speed*1000;
  Velos.y := Velos.y - temp.y*speed*1000;
  Velos.z := Velos.z - temp.z*speed*1000;

  if (((temp.x > 0) and (Velos.x < 0)) or ((temp.x < 0) and (Velos.x > 0))) then
    Velos.x := 0;
  if (((temp.y > 0) and (Velos.y < 0)) or ((temp.y < 0) and (Velos.y > 0))) then
    Velos.y := 0;
  if (((temp.z > 0) and (Velos.z < 0)) or ((temp.z < 0) and (Velos.z > 0))) then
    Velos.z := 0;

  desp := ScaleVector(Velos, speed);

  Result := ScaleVector(Normalize2(desp),Distance);

end;

Function GetPointOnLineUsingK(P1,P2 : TVector3f; K : Extended) : TVector3f;
begin
// Equation Of 3d Line: P = P1 + k(P2-P1)
Result.X := P1.X + k*(P2.X-P1.X);
Result.Y := P1.Y + k*(P2.Y-P1.Y);
Result.Z := P1.Z + k*(P2.Z-P1.Z);
end;

Function ConvertHLAngleToHelios(Angle : Single) : Single;
begin
Result := 360-(Angle - 90);
end;

Function SqrtVector(V : TVector3f) : TVector3f;
begin
Result.X := sqrt(V.X);
Result.Y := sqrt(V.Y);
Result.Z := sqrt(V.Z);
end;

Function Min3f(V,V2 : TVector3f) : Boolean; overload
begin
Result := False;

if V.X < V2.X then
if V.Y < V2.Y then
if V.Z < V2.Z then
Result := True;

end;

Function Max3f(V,V2 : TVector3f) : Boolean; overload
begin
Result := False;

if V.X > V2.X then
if V.Y > V2.Y then
if V.Z > V2.Z then
Result := True;

end;

Function Min3f2(V,V2 : TVector3f) : TVector3f; overload
begin

if V.X < V2.X then
if V.Y < V2.Y then
if V.Z < V2.Z then
Result := V
else
Result := V2;

end;

Function Max3f2(V,V2 : TVector3f) : TVector3f; overload
begin

if V.X > V2.X then
if V.Y > V2.Y then
if V.Z > V2.Z then
Result := V
else
Result := V2;

end;

Function IsVectorsDifferent(V1,V2 : TVector3f) : Boolean;
begin
Result := False;

If V1.X <> V2.X then
Result := True;

If V1.Y <> V2.Y then
Result := True;

If V1.Z <> V2.Z then
Result := True;
end;

end.
